/**
 * \addtogroup Sio_poll
 * @{
 */

/*****************************************************************************/
/**
 * \file Sio_poll.h
 * Low level 8051 serial input/ output routines for Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.6
 * \date 20-05-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#ifndef SIO_POLL_H
#define SIO_POLL_H

#include "REG51.h"
#include "Delay.h"

/** 
 * Hardware crystal frequency in Hz.
 * Adjust \b SIO_POLL_XTAL to match target hardware clock crystal
 * frequency in Hertz.
 * \hideinitializer
 */
#define SIO_POLL_XTAL             22118400

/** 
 * X2 mode support binary switch.
 * Set \b SIO_POLL_X2_MODE when CPU X2 clock mode has been elsewhere
 * engaged.
 * \hideinitializer
 */
#define SIO_POLL_X2_MODE          0

#if (SIO_POLL_X2_MODE == 1)
  /** 
   * Clock control special function register name redefinition.
   * Uncomment this redefinition for the original CKCON clock
   * control register in circumstances where your device-specific
   * SFR register declaration header file inconveniently renames
   * it as CKCON0.
   * \hideinitializer
   */
//  #define CKCON                   CKCON0
#endif

/*
 * Timer 1 mode 2 clock rate.
 */
#define SIO_POLL_T1_CLOCK         (SIO_POLL_XTAL / 12uL)  

#ifdef REG51_H
  #if (SIO_POLL_X2_MODE == 1)
    /*
     * Flag as an error, any attempt to use the X2 operating mode,
     * whilst the module still includes it's original 8051 register
     * declaration header file REG51.h. 
     */
    #error X2 operating mode requires prior substitution of generic 8051   
    #error register declaration header file Reg51.h, with a more machine   
    #error specific register declaration header file, from a device that  
    #error supports the X2 CPU clock operating mode. 
  #endif
#endif


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * Sio initialisation function.
 * Initialises the 8051 serial port for polled, specified baud-rate 8N1 
 * operation. Must be called once to set-up serial hardware prior to then
 * calling any high level functions (printf(), scanf(), etc.) that use the
 * low level serial routines, or before calling the low-level routines
 * directly.
 * \param baud_rate the selected baud-rate as an \b unsigned \b long.
 * \return -
 */
void init_sio_poll(unsigned long baud_rate);


/** 
 * Sio character transmission function.
 * The %putchar() function transmits the character \a outChar using the 8051
 * serial port, and returns the character that was output.
 * The %putchar() function is called by high level routines like puts() and
 * printf(), and may also be called directly.
 * \param outChar the \b char to transmit.
 * \return the \b char transmitted.
 */
char putchar(char outChar); 


/**
 * Sio character reception function.
 * The %_getkey() function waits for a character to be received from the serial
 * port, and then returns the received character.
 * The %_getkey() function is called by high level routines like getchar() and
 * scanf(), and may also be called directly.
 * \param -
 * \return the received \b char.
 */
char _getkey(void);


#endif

/** @} */
