/**
 * \addtogroup Sio_poll
 * @{
 */

/*****************************************************************************/
/**
 * \file Sio_poll.c
 * Low level 8051 serial input/ output routines for Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.6
 * \date 20-05-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/


#include "Sio_poll.h"

/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                              init_sio_poll()
 *
 *****************************************************************************/
void init_sio_poll(unsigned long baud_rate)
{

#if (SIO_POLL_X2_MODE == 1)

    /* 
     * Set T1X2 bit 2 of CKCON so Timer 1 clock is initially in X1 mode with
     * 12 clock periods per peripheral clock cycle.
     */
    CKCON |= 0x04;

#endif

    /* Halt Timer 1. */
    TR1  = 0;

    /* Disable Timer 1 interrupt. */
    ET1  = 0;

    /* Setup timer/ counter 1 for mode 2, 8-bit auto-reload timer. */
    TMOD &= 0x0F;
    TMOD |= 0x20;

    /* 
     * Within each of the CPU clock crystal frequency bands of interest below,
     * a baud-rate selection dependent heuristic is employed to optimally 
     * configure both UART peripheral clock and timer 1 clock operating modes.
     * All attainable baud-rate settings are exploited for the chosen clock
     * crystal frequency and CPU clock operating mode configurations.
     */

#if (SIO_POLL_XTAL < 6000000)

    /*
     * For crystal frequency selections below 6.000 MHz, UART peripheral clock
     * doubling is engaged at all baud-rates. If available here, X2 mode 
     * clocking of timer 1 is also engaged at baud-rates of 300 and above.
     */

    /* Halve requested baud-rate to accommodate doubled UART clock rate. */
    baud_rate = baud_rate >> 1;

    /* Turn SMOD1 on to double UART periph clock rate in modes 1, 2 & 3. */
    PCON |= 0x80;

  #if (SIO_POLL_X2_MODE == 1)

    if (baud_rate > 110u)  /* ie. If baud_rate originally 300 or more. */
    {
        /*
         * Halve requested baud-rate again to accommodate change of T1 clock
         * source to X2 mode.
         */
        baud_rate = baud_rate >> 1;

        /* 
         * Reset T1X2 bit 2 of CKCON so Timer 1 clock changes to X2 mode with
         * 6 clock periods per peripheral clock cycle.
         */
        CKCON &= ~0x04;
    }

  #endif

#elif (SIO_POLL_XTAL < 15000000)

    /*
     * For crystal frequency selections in the inclusive range of 6.000 MHz 
     * to 14.7456 MHz, UART peripheral clock doubling is engaged for baud-
     * rates of 300 and above. If available here, X2 mode clocking of timer 1
     * is also engaged at baud-rates of 600 and above.
     */

    if (baud_rate >= 300u)
    {
        /* Halve requested baud-rate to accommodate doubled UART clock rate. */
        baud_rate = baud_rate >> 1;

        /* Turn SMOD1 on to double UART periph clock rate in modes 1, 2 & 3. */
        PCON |= 0x80;
    }

  #if (SIO_POLL_X2_MODE == 1)

    if (baud_rate >= 300u)  /* ie. If baud_rate originally 600 or more. */
    {
        /*
         * Halve requested baud-rate again to accommodate change of T1 clock
         * source to X2 mode.
         */
        baud_rate = baud_rate >> 1;

        /* 
         * Reset T1X2 bit 2 of CKCON so Timer 1 clock changes to X2 mode with
         * 6 clock periods per peripheral clock cycle.
         */
        CKCON &= ~0x04;
    }

  #endif

#elif (SIO_POLL_XTAL < 30000000)

    /*
     * For crystal frequency selections in the inclusive range of 15.000 MHz 
     * to 28.322 MHz, UART peripheral clock doubling is engaged for baud-
     * rates of 600 and above. If available here, X2 mode clocking of timer 1
     * is also engaged at baud-rates of 1200 and above.
     */

    if (baud_rate >= 600u)
    {
        /* Halve requested baud-rate to accommodate doubled UART clock rate. */
        baud_rate = baud_rate >> 1;

        /* Turn SMOD1 on to double UART periph clock rate in modes 1, 2 & 3. */
        PCON |= 0x80;
    }

  #if (SIO_POLL_X2_MODE == 1)

    if (baud_rate >= 600u)  /* ie. If baud_rate originally 1200 or more. */
    {
        /*
         * Halve requested baud-rate again to accommodate change of T1 clock
         * source to X2 mode.
         */
        baud_rate = baud_rate >> 1;

        /* 
         * Reset T1X2 bit 2 of CKCON so Timer 1 clock changes to X2 mode with
         * 6 clock periods per peripheral clock cycle.
         */
        CKCON &= ~0x04;
    }

  #endif

#elif (SIO_POLL_XTAL <= 40000000)

    /*
     * For crystal frequency selections in the inclusive range of 30.000 MHz 
     * to 40.000 MHz, UART peripheral clock doubling is engaged for baud-
     * rates of 1200 and above. If available here, X2 mode clocking of timer 1
     * is also engaged at baud-rates of 2400 and above.
     */
    if (baud_rate >= 1200u)
    {
        /* Halve requested baud-rate to accommodate doubled UART clock rate. */
        baud_rate = baud_rate >> 1;

        /* Turn SMOD1 on to double UART periph clock rate in modes 1, 2 & 3. */
        PCON |= 0x80;
    }

  #if (SIO_POLL_X2_MODE == 1)

    if (baud_rate >= 1200u)  /* ie. If baud_rate originally 2400 or more. */
    {
        /*
         * Halve requested baud-rate again to accommodate change of T1 clock
         * source to X2 mode.
         */
        baud_rate = baud_rate >> 1;

        /* 
         * Reset T1X2 bit 2 of CKCON so Timer 1 clock changes to X2 mode with
         * 6 clock periods per peripheral clock cycle.
         */
        CKCON &= ~0x04;
    }

  #endif

#else /* (SIO_POLL_XTAL > 40000000) */

    /*
     * Flag unsupported crystal frequency configuration.
     */
    #error Crystal frequency setting exceeds program version capabilities.

#endif

    /* 
     * Calculation of timer/ counter 1 reload count is accomplished by a 
     * method chosen to produce the most optimally economical object code size. 
     * This initially unwieldy looking marshalled timer reload constant
     * calculation block, optimises to yield a surprisingly efficient result.
     */
    if ((baud_rate > sizeof(unsigned int)) 
                       && ((SIO_POLL_T1_CLOCK / 32u) > sizeof(unsigned int)))
    {
        /* 
         * Quarter requested baud-rate to avoid an object-code-bloating
         * 32-bit division. 
         */
        baud_rate = baud_rate >> 2;

        /* Calculate timer/ counter 1 reload count. */
        TH1  = 256u - (unsigned char)((unsigned int)(SIO_POLL_T1_CLOCK / 128uL)
                                                    / (unsigned int)baud_rate);
    }
    else if ((baud_rate > sizeof(unsigned int)) 
                       || ((SIO_POLL_T1_CLOCK / 32u) > sizeof(unsigned int)))
    {
        /* 
         * Halve requested baud-rate to avoid an object-code-bloating
         * 32-bit division. 
         */
        baud_rate = baud_rate >> 1;

        /* Calculate timer/ counter 1 reload count. */
        TH1  = 256u - (unsigned char)((unsigned int)(SIO_POLL_T1_CLOCK / 64uL)
                                                  / (unsigned int)baud_rate);
    }
    else
    {
        /* Calculate timer/ counter 1 reload count. */
        TH1  = 256u - (unsigned char)((unsigned int)(SIO_POLL_T1_CLOCK / 32uL)
                                                   / (unsigned int)baud_rate);
    }

    /* Restart timer/ counter 1, so it may free run. */
    TR1 = 1;

    /* Set serial port for mode 1, 8N1 operation. */
    SCON = 0x50;

    /* Set TI to indicate TX buffer empty. */
    TI = 1;

    /* 
     * Allow hardware to briefly settle before permitting use of it. Omit this
     * step to your own eventual peril.
     */
    delay_ms(10);

    return;
}


/*****************************************************************************
 *
 *                                 putchar()
 *
 *****************************************************************************/
char putchar(char outChar)
{
    /* Wait 'till previous character sent. */
    while(!TI) continue;

    /* Clear TI to indicate TX buffer full. */
    TI = 0;

    /* Place next character to send into sio. */
    SBUF = outChar;

    return outChar;
}


/*****************************************************************************
 *
 *                                 _getkey()
 *
 *****************************************************************************/
char _getkey(void)
{
    char inChar;

    /* Wait until a character has been received. */
    while(!RI) continue;

    /* Read received character from sio. */
    inChar = SBUF;

    /* Clear RI to indicate RX buffer empty. */
    RI = 0;

    return inChar;
}

/** @} */
